@@ -7,11 +7,11 @@ from time import sleep, time
 from ...PILogger import PIDebug
 from ...pidevice import GCS30Commands
 from ...pidevice.common.gcscommands_helpers import isdeviceavailable, getitemsvaluestuple
-from ...pidevice.gcs30.gcs30commands_helpers import PIAxisStatusKeys
+from ...pidevice.gcs30.gcs30commands_helpers import PIAxisStatusKeys, PIControlModes
 from ...pidevice.gcs30.gcs30error import GCS30Error
 from ...pitools.common.gcsbasepitools import GCSBaseDeviceStartup, GCSBaseTools
 
-__signature__ = 0xcc8a7ad021ecebde6ca4d4e0f54677f4
+__signature__ = 0xb8826aff8f8f452c747316289348e45f
 
 
 # Class inherits from object, can be safely removed from bases in python3 pylint: disable=R0205
@@ -61,9 +61,12 @@ class GCS30DeviceStartup(GCSBaseDeviceStartup):  # Too many instance attributes
 
         if isinstance(self._controlmodes, int):
             servostates = \
-                [True] * self._pidevice.numaxes if self._controlmodes == 0x2 else [False] * self._pidevice.numaxes
+                [True] * self._pidevice.numaxes \
+                    if self._controlmodes ==  PIControlModes.CLOSED_LOOP_POSITION.value \
+                    else [False] * self._pidevice.numaxes
         else:
-            servostates = [bool(n == 0x2) for n in self._controlmodes]
+            servostates = \
+                [n == PIControlModes.CLOSED_LOOP_POSITION.value for n in self._controlmodes]
 
         return  dict(list(zip(self._pidevice.axes, servostates)))
 
@@ -73,9 +76,14 @@ class GCS30DeviceStartup(GCSBaseDeviceStartup):  # Too many instance attributes
         if servo is None:
             self._controlmodes = None
         elif isinstance(servo, bool):
-            self._controlmodes = 0x2 if servo is True else 0x1
+            self._controlmodes = PIControlModes.CLOSED_LOOP_POSITION.value \
+                if servo is True \
+                else PIControlModes.OPEN_LOOP_POSITION.value
         else:
-            self._controlmodes = {a:0x1 if servo[a] is False else 0x2 for a in servo.keys()}
+            self._controlmodes = {a:PIControlModes.OPEN_LOOP_POSITION.value
+                if servo[a] is False
+                else PIControlModes.CLOSED_LOOP_POSITION.value
+                                  for a in servo.keys()}
 
         PIDebug('GCS30DeviceStartup.servostates = %s', self._pitools.itemstostr(self._controlmodes))
 
@@ -274,7 +282,7 @@ class GCS30Tools(GCSBaseTools):  # Too  public methods pylint: disable=R0903
 
     def _get_servo_state(self, axes):
         control_modes = self._get_control_mode(axes)
-        return  {a: bool(control_modes[a] == 0x2) for a in control_modes.keys()}
+        return  {a: bool(control_modes[a] in  PIControlModes.CLOSED_LOOP.value) for a in control_modes.keys()}
 
     def _read_axis_status_flag(self, axes, flag, defaultvalue=False, throwonaxiserror=False):
         axes_status_flag = self._pidevice.get_axes_status_flag(axes, flag, throwonaxiserror)
@@ -305,7 +313,13 @@ class GCS30Tools(GCSBaseTools):  # Too  public methods pylint: disable=R0903
                                            throwonaxiserror=throwonaxiserror)
 
     def _get_open_loop_on_target(self, axes):
-        return dict(list(zip(axes, [True] * len(axes))))
+        isontarget = {}
+        if self._pidevice.HasqSMR():
+            stepsleft = self._get_open_loop_remaining_steps(axes).values()
+            isontarget.update(dict(list(zip(axes, [x == 0 for x in stepsleft]))))
+        else:
+            isontarget.update(dict(list(zip(axes, [True] * len(axes)))))
+        return isontarget
 
     # Too many arguments pylint: disable=R0913
     def _wait_to_the_end_of_reference(self, axes, timeout, polldelay):
@@ -319,3 +333,20 @@ class GCS30Tools(GCSBaseTools):  # Too  public methods pylint: disable=R0903
     def _enable_axes(self, axes):
         for axis in axes:
             self._pidevice.EAX(axis, True)
+
+    def _get_open_loop_remaining_steps(self, axes):
+        axes = self.getaxeslist(axes)
+        answer = dict(list(zip(axes, [0] * len(axes))))
+        if self._pidevice.HasqSMR():
+            if len(axes) == 1:
+                remaining_steps = self._pidevice.qSMR(axes)
+            else:
+                remaining_steps = self._pidevice.qSMR()
+
+            for axis in remaining_steps:
+                if not axis in axes:
+                    continue
+
+                answer[axis] = remaining_steps[axis]
+
+        return answer
